@@ -3,7 +3,7 @@
  * Author: Brent Hendricks
  * Project: libeasysock
  * Date: 4/16/98
- * $Id: easysock.c 7877 2006-02-27 12:15:43Z josef $
+ * $Id: easysock.c 8506 2006-08-08 20:34:08Z jdorje $
  *
  * A library of useful routines to make life easier while using 
  * sockets
@@ -146,7 +146,7 @@ int ggz_init_network(void)
 
 #ifdef HAVE_WINSOCK2_H
 		WSADATA wsa;
-		if (WSAStartup(MAKEWORD(1, 1), &wsa) !=0 ){
+		if (WSAStartup(MAKEWORD(2, 2), &wsa) !=0 ){
 			return -1;
 		}
 #endif
@@ -165,6 +165,9 @@ static int es_bind(const char *host, int port)
 	int sockfd;
 	const int on = 1;
 	/* FIXME: better test on getaddrinfo() directly */
+	int n;
+	struct addrinfo hints, *res, *ressave;
+	char serv[30];
 #ifdef HAVE_WINSOCK2_H
 	struct sockaddr_in name;
 
@@ -182,7 +185,7 @@ static int es_bind(const char *host, int port)
 
 	memset(&name, 0, sizeof(name));
 	name.sin_family = AF_INET;
-	name.sin_port = port;
+	name.sin_port = htons(port);
 	name.sin_addr.s_addr = htonl(INADDR_ANY);
 
 	if (bind(sockfd, &name, sizeof(name)) != 0) {
@@ -194,9 +197,6 @@ static int es_bind(const char *host, int port)
 		sockfd = -1;
 	}
 #else
-	int n;
-	struct addrinfo hints, *res, *ressave;
-	char serv[30];
 
 	snprintf(serv, sizeof(serv), "%d", (unsigned int)port);
 
@@ -265,7 +265,7 @@ static int es_connect(const char *host, int port)
 
 	memset(&name, 0, sizeof(name));
 	name.sin_family = AF_INET;
-	name.sin_port = port;
+	name.sin_port = htons(port);
 	memcpy(&name.sin_addr, h->h_addr, h->h_length);
 
 	if (connect(sockfd, &name, sizeof(name)) != 0) {
@@ -456,7 +456,8 @@ int ggz_read_char(const int sock, char *message)
 	int status;
 
 	if ( (status = ggz_readn(sock, message, sizeof(char))) < 0) {
-		ggz_debug(GGZ_SOCKET_DEBUG, "Error receiving char.");
+		ggz_debug(GGZ_SOCKET_DEBUG, "Error receiving char: %s",
+			  strerror(errno));
 		if (_err_func)
 			(*_err_func) (strerror(errno), GGZ_IO_READ, sock, GGZ_DATA_CHAR);
 		return -1;
@@ -517,7 +518,8 @@ int ggz_read_int(const int sock, int *message)
 	int data, status;
 	
 	if ( (status = ggz_readn(sock, &data, sizeof(int))) < 0) {
-		ggz_debug(GGZ_SOCKET_DEBUG, "Error receiving int.");
+		ggz_debug(GGZ_SOCKET_DEBUG, "Error receiving int: %s",
+			  strerror(errno));
 		if (_err_func)
 			(*_err_func) (strerror(errno), GGZ_IO_READ, sock, GGZ_DATA_INT);
 		return -1;
@@ -617,7 +619,8 @@ int ggz_read_string(const int sock, char *message, const unsigned int len)
 	}
 	
        	if ( (status = ggz_readn(sock, message, size)) < 0) {
-		ggz_debug(GGZ_SOCKET_DEBUG, "Error receiving string.");
+		ggz_debug(GGZ_SOCKET_DEBUG, "Error receiving string: %s",
+			  strerror(errno));
 		if (_err_func)
 			(*_err_func) (strerror(errno), GGZ_IO_READ, sock, GGZ_DATA_STRING);
 		return -1;
@@ -669,7 +672,8 @@ int ggz_read_string_alloc(const int sock, char **message)
 	*message = ggz_malloc((size+1) * sizeof(char));
 
 	if ( (status = ggz_readn(sock, *message, size)) < 0) {
-		ggz_debug(GGZ_SOCKET_DEBUG, "Error receiving string.");
+		ggz_debug(GGZ_SOCKET_DEBUG, "Error receiving string: %s",
+			  strerror(errno));
 		if (_err_func)
 			(*_err_func) (strerror(errno), GGZ_IO_READ, sock, GGZ_DATA_STRING);
 		return -1;
@@ -714,7 +718,20 @@ int ggz_writen(const int sock, const void *vptr, size_t n)
 		nwritten = write(sock, ptr, nleft);
 #endif
 		if (nwritten <= 0) {
-			if (errno == EINTR)
+			if (
+#ifdef HAVE_WINSOCK2_H
+			    /* FIXME: Somehow the socket is created
+			       nonblocking under windows, and WSAWOULDBLOCK
+			       is being returned.  The code as it is just
+			       busywaits waiting for data, which is
+			       extremely bad.  We need to make the socket
+			       blocking. */
+			    (WSAGetLastError() == WSAEINTR
+			     || WSAGetLastError() == WSAEWOULDBLOCK)
+#else
+			    errno == EINTR
+#endif
+			    )
 				nwritten = 0;	/* and call write() again */
 			else
 				return (-1);	/* error */
@@ -745,7 +762,20 @@ int ggz_readn(const int sock, void *vptr, size_t n)
 		nread = read(sock, ptr, nleft);
 #endif
 		if (nread < 0) {
-			if (errno == EINTR)
+			if (
+#ifdef HAVE_WINSOCK2_H
+			    /* FIXME: Somehow the socket is created
+			       nonblocking under windows, and WSAWOULDBLOCK
+			       is being returned.  The code as it is just
+			       busywaits waiting for data, which is
+			       extremely bad.  We need to make the socket
+			       blocking. */
+			    (WSAGetLastError() == WSAEINTR
+			     || WSAGetLastError() == WSAEWOULDBLOCK)
+#else
+			    errno == EINTR
+#endif
+			    )
 				nread = 0;	/* and call read() again */
 			else
 				return (-1);
