@@ -2,7 +2,7 @@
  * @file   ggz.h
  * @author Brent M. Hendricks
  * @date   Fri Nov  2 23:32:17 2001
- * $Id: ggz.h 7907 2006-03-14 13:49:22Z josef $
+ * $Id: ggz.h 8945 2007-01-16 17:42:57Z josef $
  * 
  * Header file for ggz components lib
  *
@@ -29,8 +29,8 @@
 
 #define LIBGGZ_VERSION_MAJOR 0
 #define LIBGGZ_VERSION_MINOR 0
-#define LIBGGZ_VERSION_MICRO 13
-#define LIBGGZ_VERSION_IFACE "4:0:2"
+#define LIBGGZ_VERSION_MICRO 14
+#define LIBGGZ_VERSION_IFACE "5:0:3"
 
 #include <sys/types.h>
 
@@ -53,12 +53,12 @@ extern "C" {
 #endif
 
 #if defined __GNUC__ && (__GNUC__ >= 3)
-#  define _GGZFUNCTION_
+#  define _GGZFUNCTION_ ""
 #else
 #  ifndef __cplusplus
 #    define _GGZFUNCTION_ __FUNCTION__
 #  else
-#    define _GGZFUNCTION_
+#    define _GGZFUNCTION_ ""
 #  endif
 #endif
 
@@ -1087,6 +1087,9 @@ char * ggz_xml_escape(const char *str);
  * expected to later free this memory using ggz_free().  If the original string
  * did not contain any characters which required decoding, a ggz_strdup() copy
  * is returned.
+ * @note When using expat, incoming text is automatically unescaped by the
+ * expat library.  It is therefore generally not necessary to use this function
+ * with expat.
  */
 char * ggz_xml_unescape(const char *str);
 
@@ -1688,6 +1691,18 @@ hash_t ggz_hash_create(const char *algo, const char *text);
  */
 hash_t ggz_hmac_create(const char *algo, const char *text, const char *secret);
 
+/** @brief Encodes text to base16.
+ *
+ *  Plain text with possibly unsafe characters is converted
+ *  to the base16 (hex) format through this function.
+ *  The returned string is allocated internally and must be freed.
+ *
+ *  @param text Plain text to encode
+ *  @param length Length of the text (which may contain binary characters), in bytes
+ *  @return Base16 representation of the text
+ */
+char *ggz_base16_encode(const char *text, int length);
+
 /** @brief Encodes text to base64.
  *
  *  Plain text with possibly unsafe characters is converted
@@ -1749,13 +1764,26 @@ void ggz_tls_init(const char *certfile, const char *keyfile, const char *passwor
 
 /** @brief Check TLS support.
  *
- *  Since handshakes can fail, this function checks whether TLS
- *  was actually successfully activated.
+ *  Checks if real TLS support is available or communication
+ *  will fall back to unencrypted connections.
+ *  Even in the case of support, individual connections might
+ *  still be unencrypted if the handshake fails.
  *
  *  @return 1 if TLS is supported, 0 if no support is present
+ *  @see ggz_tls_enable_fd
  */
 int ggz_tls_support_query(void);
 
+/** @brief Name of the TLS implementation.
+ *
+ *  Returns the name of the TLS layer implementation used
+ *  to encrypt connections.
+ *
+ *  @return TLS implementation name, or NULL if no TLS support is present
+ *  @see ggz_tls_support_query
+ */
+const char *ggz_tls_support_name(void);
+
 /** @brief Enable TLS for a file descriptor.
  *
  *  A TLS handshake is performed for an existing connection on the given
