@@ -3,7 +3,7 @@
  * Author: GGZ Dev Team
  * Project: GGZ Common Library
  * Date: 01/13/2002
- * $Id: ggz_common.c 7885 2006-03-06 18:46:12Z josef $
+ * $Id: ggz_common.c 8762 2006-12-27 10:00:21Z jdorje $
  *
  * This provides GGZ-specific functionality that is common to
  * some or all of the ggz-server, game-server, ggz-client, and
@@ -36,6 +36,9 @@
 
 #include "support.h"
 
+#define ARRAY_SIZE(arr) (sizeof(arr) / sizeof((arr)[0]))
+
+/* These are defined by the network protocol */
 #define OPEN_SEAT_NAME "open"
 #define BOT_SEAT_NAME "bot"
 #define PLAYER_SEAT_NAME "player"
@@ -87,6 +90,7 @@ GGZSeatType ggz_string_to_seattype(const char *type_str)
 	return GGZ_SEAT_NONE;
 }
 
+/* These are defined by the network protocol */
 #define NORMAL_CHAT_NAME "normal"
 #define ANNOUNCE_CHAT_NAME "announce"
 #define BEEP_CHAT_NAME "beep"
@@ -137,6 +141,50 @@ GGZChatType ggz_string_to_chattype(const char *type_str)
 	return GGZ_CHAT_UNKNOWN;
 }
 
+/* These are defined by the network protocol */
+#define GAG_ADMIN_NAME "gag"
+#define UNGAG_ADMIN_NAME "ungag"
+#define KICK_ADMIN_NAME "kick"
+#define BAN_ADMIN_NAME "ban"
+
+const char *ggz_admintype_to_string(GGZAdminType type)
+{
+	switch (type) {
+	case GGZ_ADMIN_GAG:
+		return GAG_ADMIN_NAME;
+	case GGZ_ADMIN_UNGAG:
+		return UNGAG_ADMIN_NAME;
+	case GGZ_ADMIN_KICK:
+		return KICK_ADMIN_NAME;
+	case GGZ_ADMIN_BAN:
+		return BAN_ADMIN_NAME;
+	case GGZ_ADMIN_UNKNOWN:
+		break;
+	}
+
+	ggz_error_msg("ggz_admintype_to_string: "
+		      "invalid admintype %d given.", type);
+	return ""; /* ? */
+}
+
+GGZAdminType ggz_string_to_admintype(const char *type_str)
+{
+	if (!type_str)
+		return GGZ_ADMIN_UNKNOWN;
+
+	if (strcasecmp(type_str, GAG_ADMIN_NAME) == 0)
+		return GGZ_ADMIN_GAG;
+	else if (strcasecmp(type_str, UNGAG_ADMIN_NAME) == 0)
+		return GGZ_ADMIN_UNGAG;
+	else if (strcasecmp(type_str, KICK_ADMIN_NAME) == 0)
+		return GGZ_ADMIN_KICK;
+	else if (strcasecmp(type_str, BAN_ADMIN_NAME) == 0)
+		return GGZ_ADMIN_BAN;
+
+	return GGZ_ADMIN_UNKNOWN;
+}
+
+/* These are defined by the network protocol */
 #define NORMAL_LEAVETYPE_NAME "normal"
 #define BOOT_LEAVETYPE_NAME "boot"
 #define GAMEOVER_LEAVETYPE_NAME "gameover"
@@ -177,9 +225,11 @@ GGZLeaveType ggz_string_to_leavetype(const char *type_str)
 	return GGZ_LEAVE_GAMEERROR;
 }
 
+/* These are defined by the network protocol */
 #define GUEST_PLAYER_NAME "guest"
 #define NORMAL_PLAYER_NAME "normal"
 #define ADMIN_PLAYER_NAME "admin"
+#define HOST_PLAYER_NAME "host"
 #define BOT_PLAYER_NAME "bot"
 #define UNKNOWN_PLAYER_NAME "unknown"
 
@@ -192,6 +242,8 @@ const char *ggz_playertype_to_string(GGZPlayerType type)
 		return NORMAL_PLAYER_NAME;
 	case GGZ_PLAYER_ADMIN:
 		return ADMIN_PLAYER_NAME;
+	case GGZ_PLAYER_HOST:
+		return HOST_PLAYER_NAME;
 	case GGZ_PLAYER_BOT:
 		return BOT_PLAYER_NAME;
 	case GGZ_PLAYER_UNKNOWN:
@@ -215,6 +267,8 @@ GGZPlayerType ggz_string_to_playertype(const char *type_str)
 		return GGZ_PLAYER_GUEST;
 	} else if (strcasecmp(type_str, ADMIN_PLAYER_NAME) == 0) {
 		return GGZ_PLAYER_ADMIN;
+	} else if (strcasecmp(type_str, HOST_PLAYER_NAME) == 0) {
+		return GGZ_PLAYER_HOST;
 	} else if (strcasecmp(type_str, BOT_PLAYER_NAME) == 0) {
 		return GGZ_PLAYER_BOT;
 	}
@@ -222,6 +276,7 @@ GGZPlayerType ggz_string_to_playertype(const char *type_str)
 	return GGZ_PLAYER_UNKNOWN; /* ? */
 }
 
+/* These are defined by the network protocol */
 #define TRUE_NAME "true"
 #define FALSE_NAME "false"
 
@@ -277,8 +332,8 @@ const char *ggz_error_to_string(GGZClientReqError err)
 		return "leave forbidden";
 	case E_ALREADY_LOGGED_IN:
 		return "already logged in";
-	case E_NOT_LOGGED_IN
-		: return "not logged in";
+	case E_NOT_LOGGED_IN:
+		return "not logged in";
 	case E_NOT_IN_ROOM:
 		return "not in room";
 	case E_AT_TABLE:
@@ -297,6 +352,12 @@ const char *ggz_error_to_string(GGZClientReqError err)
 		return "too long";
 	case E_BAD_USERNAME:
 		return "bad username";
+	case E_USR_TYPE:
+		return "wrong login type";
+	case E_USR_FOUND:
+		return "user not found";
+	case E_USR_TAKEN:
+		return "username already taken";
 	case E_NO_STATUS:
 	case E_UNKNOWN:
 		break;
@@ -360,6 +421,107 @@ GGZClientReqError ggz_string_to_error(const char *str)
 		return E_TOO_LONG;
 	if (!strcasecmp(str, "bad username"))
 		return E_BAD_USERNAME;
+	if (!strcasecmp(str, "wrong login type"))
+		return E_USR_TYPE;
+	if (!strcasecmp(str, "user not found"))
+		return E_USR_FOUND;
+	if (!strcasecmp(str, "username already taken"))
+		return E_USR_TAKEN;
 
 	return E_UNKNOWN;
 }
+
+
+bool ggz_perms_is_host(GGZPermset perms)
+{
+	return (ggz_perms_is_set(perms, GGZ_PERM_EDIT_TABLES)
+		|| ggz_perms_is_set(perms, GGZ_PERM_TABLE_PRIVMSG));
+}
+
+
+bool ggz_perms_is_bot(GGZPermset perms)
+{
+	return ggz_perms_is_set(perms, GGZ_PERM_CHAT_BOT);
+}
+
+
+bool ggz_perms_is_admin(GGZPermset perms)
+{
+	return (ggz_perms_is_set(perms, GGZ_PERM_ROOMS_ADMIN)
+		&& ggz_perms_is_set(perms, GGZ_PERM_CHAT_ANNOUNCE)
+		&& ggz_perms_is_set(perms, GGZ_PERM_EDIT_TABLES)
+		&& ggz_perms_is_set(perms, GGZ_PERM_TABLE_PRIVMSG));
+}
+
+
+void ggz_perms_init_from_list(GGZPermset *perms, GGZPerm *list, size_t listsz)
+{
+	int i;
+
+	*perms = 0;
+	for (i = 0; i < listsz; i++) {
+		*perms |= (1 << list[i]);
+	}
+}
+
+
+void ggz_perms_set(GGZPermset *perms, GGZPerm perm, bool val)
+{
+	if (perm >= 0 || perm < GGZ_PERM_COUNT) {
+		if (val) {
+			/* Set bit */
+			(*perms) |= (1 << perm);
+		} else {
+			/* Clear bit. */
+			(*perms) &= ~(1 << perm);
+		}
+	}
+	/* Else fail silently. */
+}
+
+
+bool ggz_perms_is_set(GGZPermset perms, GGZPerm perm)
+{
+	return (perm >= 0 && perm < GGZ_PERM_COUNT
+		&& (perms & (1 << perm)) != 0);
+}
+
+
+/* These are defined by the network protocol */
+static const char *perm_names[] = {
+	"join_table",
+	"launch_table",
+	"rooms_login",
+	"rooms_admin",
+	"chat_announce",
+	"chat_bot",
+	"no_stats",
+	"edit_tables",
+	"edit_privmsg"
+};
+
+
+const char *ggz_perm_to_string(GGZPerm perm)
+{
+	if (perm >= 0 && perm < ARRAY_SIZE(perm_names)) {
+		return perm_names[perm];
+	} else {
+		return "(UNKNOWN)";
+	}
+}
+
+
+GGZPerm ggz_string_to_perm(const char *perm_str)
+{
+	GGZPerm p;
+
+	if (!perm_str) return GGZ_PERM_COUNT;
+
+	for (p = 0; p < GGZ_PERM_COUNT; p++) {
+		if (strcasecmp(perm_str, perm_names[p]) == 0) {
+			return p;
+		}
+	}
+
+	return GGZ_PERM_COUNT;
+}
