@@ -3,7 +3,7 @@
  * Author: GGZ Dev Team
  * Project: GGZ Common Library
  * Date: 01/13/2002
- * $Id: ggz_common.h 7885 2006-03-06 18:46:12Z josef $
+ * $Id: ggz_common.h 8759 2006-12-27 04:02:24Z jdorje $
  *
  * This provides GGZ-specific functionality that is common to
  * some or all of the ggz-server, game-server, ggz-client, and
@@ -29,6 +29,9 @@
 #ifndef __GGZ_COMMON_H__
 #define __GGZ_COMMON_H__
 
+#include <stdbool.h>
+#include <sys/types.h>
+
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
@@ -114,6 +117,19 @@ typedef enum {
 	GGZ_LEAVE_GAMEERROR
 } GGZLeaveType;
 
+/** @brief Administrative actions.
+ *
+ * Administrators or to some extent hosts are able to dissolve player
+ * problems by several levels of punishment.
+ */
+typedef enum {
+	GGZ_ADMIN_GAG,		/**< Player chat is ignored by all others */
+	GGZ_ADMIN_UNGAG,	/**< Reversion of temporary gagging */
+	GGZ_ADMIN_KICK,		/**< Player is kicked from the server */
+	GGZ_ADMIN_BAN,		/**< Player banned permanently (unused) */
+	GGZ_ADMIN_UNKNOWN	/**< Invalid admin type */
+} GGZAdminType;
+
 /** @brief A player type.
  *
  *  The server will tell the client the type of each player.
@@ -125,9 +141,12 @@ typedef enum {
 	/** A guest player is not registered. */
 	GGZ_PLAYER_GUEST,
 
-	/* An admin player is registered and has some special permissions. */
+	/* An admin player is registered and has all special permissions. */
 	GGZ_PLAYER_ADMIN,
 
+	/* A host player is registered and has a few special permissions. */
+	GGZ_PLAYER_HOST,
+
 	/* A bot is a special type of player. */
 	GGZ_PLAYER_BOT,
 
@@ -215,6 +234,25 @@ const char *ggz_playertype_to_string(GGZPlayerType type);
  */
 GGZPlayerType ggz_string_to_playertype(const char *type_str);
 
+/** @brief Get a string identifier for the GGZAdminType.
+ *
+ *  This returns a pointer to a static string describing the given admin
+ *  action.  It is useful for text-based communications protocols and
+ *  debugging output.
+ *  @param op The GGZAdminType, which determines the string returned.
+ *  @note This is the inverse of ggz_string_to_admintype.
+ */
+const char *ggz_admintype_to_string(GGZAdminType type);
+
+/** @brief Get a GGZAdminType for the given string identifier.
+ *
+ *  This returns a GGZAdminType that is associated with the given string
+ *  description.
+ *  @param type_str A string describing a GGZAdminType.
+ *  @note This is the inverse of ggz_admintype_to_string.
+ */
+GGZAdminType ggz_string_to_admintype(const char *type_str);
+
 
 /** @brief Convert a string to a boolean.
  * 
@@ -303,7 +341,10 @@ typedef enum {
 	E_NO_CHANNEL        = -19,
 	E_TOO_LONG          = -20,
 	E_UNKNOWN           = -21,
-	E_BAD_USERNAME      = -22
+	E_BAD_USERNAME      = -22,
+	E_USR_TYPE          = -23,
+	E_USR_FOUND         = -24,
+	E_USR_TAKEN         = -25
 } GGZClientReqError;
 
 /** @brief Get a string identifier for the GGZClientReqError.
@@ -327,6 +368,73 @@ const char *ggz_error_to_string(GGZClientReqError err);
 GGZClientReqError ggz_string_to_error(const char *err_str);
 
 
+
+/**
+ * @defgroup perms Player permissions
+ *
+ * These definitions and functions provide a set of permissions that GGZ
+ * players may have.
+ */
+
+/** @brief Boolean permissions for GGZ players. */
+typedef enum {
+	GGZ_PERM_JOIN_TABLE,	/**< Can join a table */
+	GGZ_PERM_LAUNCH_TABLE,	/**< Can launch a new table */
+	GGZ_PERM_ROOMS_LOGIN,	/**< Can enter login-only rooms */
+	GGZ_PERM_ROOMS_ADMIN,	/**< Can enter admin-only rooms */
+	GGZ_PERM_CHAT_ANNOUNCE,	/**< Can make announcements */
+	GGZ_PERM_CHAT_BOT,	/**< Player is a known bot */
+	GGZ_PERM_NO_STATS,	/**< No stats for this player. */
+	GGZ_PERM_EDIT_TABLES,	/**< Can edit tables. */
+	GGZ_PERM_TABLE_PRIVMSG,	/**< Can send private messages at a table */
+	GGZ_PERM_COUNT		/**< Placeholder */
+} GGZPerm;
+
+/** @brief A permission set (bitfield) contains all perms for one player. */
+typedef unsigned int GGZPermset;
+
+/** @brief Return true if the perm set qualifies the player as a 'host' */
+bool ggz_perms_is_host(GGZPermset perms);
+
+/** @brief Return true if the perm set qualifies the player as a 'bot' */
+bool ggz_perms_is_bot(GGZPermset perms);
+
+/** @brief Return true if the perm set qualifies the player as an 'admin' */
+bool ggz_perms_is_admin(GGZPermset perms);
+
+/** @brief Initialize the perm set from the given list array. */
+void ggz_perms_init_from_list(GGZPermset *perms,
+			      GGZPerm *list, size_t listsz);
+
+/** @brief Sets/clears the given permission.
+ *  @param perms The permissions set to modify.
+ *  @param perm The permission in the set to change.
+ *  @param value Whether to set or clear the permission from the set. */
+void ggz_perms_set(GGZPermset *perms, GGZPerm perm, bool val);
+
+/** @brief Return true if the permissions is in the set. */
+bool ggz_perms_is_set(GGZPermset perms, GGZPerm perm);
+
+/** @brief Get a string identifier for the GGZPerm.
+ *
+ *  This returns a pointer to a static string describing the given permission.
+ *  It is useful for text-based communications protocols and debugging
+ *  output.
+ *  @param perm The GGZPerm, which determines the string returned.
+ *  @note This is the inverse of ggz_string_to_perm.
+ */
+const char *ggz_perm_to_string(GGZPerm perm);
+
+/** @brief Get a GGZPerm for the given string identifier.
+ *
+ *  This returns a GGZPerm that is associated with the given string
+ *  description.
+ *  @param perm_str A string describing a GGZPerm.
+ *  @note If the perm_str cannot be parsed GGZ_PERM_COUNT may be returned.
+ *  @note This is the inverse of ggz_perm_to_string.
+ */
+GGZPerm ggz_string_to_perm(const char *perm_str);
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
